from daktari.config import read_config
from daktari.check_runner import run_checks
from daktari.options import argument_parser
import sys
import logging
from pyfiglet import Figlet
import os


def print_logo(title: str):
    figlet = Figlet(font="slant")
    print(figlet.renderText(title))


def main() -> int:
    args = argument_parser.parse_args()
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    config = read_config(args.config_path)

    os.chdir(args.config_path.parent.absolute())

    if config.title:
        print_logo(config.title)
    all_passed = run_checks(config.checks)
    return 0 if all_passed else 1


if __name__ == "__main__":
    sys.exit(main())



import distro


class OS:
    OS_X = "osx"
    GENERIC = "generic"
    UBUNTU = "ubuntu"


def detect_os() -> str:
    (id_name, _, _) = distro.linux_distribution()
    if id_name == "Darwin":
        return OS.OS_X
    elif id_name == "Ubuntu":
        return OS.UBUNTU
    else:
        return OS.GENERIC



from typing import Dict, List, Set, TypeVar
from daktari.check import Check


def dependency_graph(checks: List[Check]) -> Dict[str, Set[str]]:
    graph: Dict[str, Set[str]] = {}
    for check in checks:
        graph[check.name] = set()
        for dependency in check.depends_on:
            graph[check.name] = graph[check.name].union({dependency.name})
    return graph


T = TypeVar("T")


def stable_topological_sort(items: List[T], dependencies: Dict[T, Set[T]]) -> List[T]:
    result = items.copy()
    for i in range(len(items)):
        for j in range(i):
            if result[i] in dependencies[result[j]]:
                item = result[i]
                del result[i]
                result.insert(j, item)
    return result


def sort_checks(checks: List[Check]) -> List[Check]:
    graph = dependency_graph(checks)
    check_names = [check.name for check in checks]
    sorted_check_names = stable_topological_sort(check_names, graph)

    def get_topo_order(check: Check):
        return sorted_check_names.index(check.name)

    return sorted(checks, key=get_topo_order)



import re
from daktari.os import OS, detect_os
from daktari.check import CheckResult, CheckStatus
from typing import Dict, Optional
from colors import red, green, underline


def check_status_symbol(status: CheckStatus) -> str:
    return "✅" if status == CheckStatus.PASS else "❌"


def get_most_specific_suggestion(this_os: str, suggestions: Dict[str, str]) -> Optional[str]:
    def get_suggestion(os_to_match: str) -> Optional[str]:
        return next((message for os, message in suggestions.items() if os == os_to_match), None)

    return get_suggestion(this_os) or get_suggestion(OS.GENERIC)


def print_suggestion_text(text: str):
    pattern = re.compile("<cmd>(.+)</cmd>")

    def add_ansi_underline(match):
        return underline(match.group(1))

    replaced = pattern.sub(add_ansi_underline, text)
    lines = [line.lstrip() for line in replaced.splitlines()]

    raw_lines = [line.lstrip() for line in re.compile("</?cmd>").sub("", text).splitlines()]

    max_width = max([len(line) for line in raw_lines])
    title = "💡 Suggestion "
    print("┌─" + title + "─" * (max_width - len(title)) + "┐")
    for i, line in enumerate(lines):
        raw_line = raw_lines[i]
        padding = " " * (max_width - len(raw_line))
        print(f"│ {line}{padding} │")
    print("└" + "─" * (max_width + 2) + "┘")


def print_check_result(result: CheckResult):
    this_os = detect_os()
    status_symbol = check_status_symbol(result.status)
    colour = green if result.status == CheckStatus.PASS else red
    print(f"{status_symbol} [{colour(result.name)}] {result.summary}")
    if result.status == CheckStatus.FAIL:
        suggestion = get_most_specific_suggestion(this_os, result.suggestions)
        if suggestion:
            print_suggestion_text(suggestion)



from daktari.os import OS
from daktari.check import Check, CheckResult
from daktari.command_utils import can_run_command


class WatchmanInstalled(Check):
    name = "watchman.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install watchman</cmd>",
        OS.GENERIC: "Install watchman: https://facebook.github.io/watchman/docs/install.html#buildinstall",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("watchman --version"), "Watchman is <not/> installed")


class MkcertInstalled(Check):
    name = "mkcert.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install mkcert</cmd>",
        OS.GENERIC: "Install mkcert: https://mkcert.dev/#installation",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("mkcert -version"), "mkcert is <not/> installed")



import setuptools
import pathlib

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setuptools.setup(
    name="daktari",
    version="0.0.3",
    description="Assist in setting up and maintaining developer environments",
    long_description=README,
    long_description_content_type="text/markdown",
    license="MIT",
    author="Matt Russell",
    author_email="matthew.russell@sonocent.com",
    url="https://github.com/sonocent/daktari",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    python_requires=">=3.6",
    entry_points={
        "console_scripts": [
            "daktari = daktari.__main__:main",
        ],
    },
    include_package_data=True,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
    ],
)
